#include "graph.h"

/*
 *  Constructs a GraphDlg as a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
GraphDlg::GraphDlg( QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )

{
    if ( !name )
	setName( "GraphDlg" );
    setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, sizePolicy().hasHeightForWidth() ) );
    setMinimumSize( QSize( 600, 480 ) );
    setMaximumSize( QSize( 600, 480 ) );

    groupBox1 = new QGroupBox( this, "groupBox1" );
    groupBox1->setGeometry( QRect( 20, 0, 561, 240 ) );
    groupBox1->setFrameShape( QGroupBox::GroupBoxPanel );
    groupBox1->setFrameShadow( QGroupBox::Sunken );

    HeightLabel = new QLabel( groupBox1, "HeightLabel" );
    HeightLabel->setGeometry( QRect( 40, 170, 60, 21 ) );
    HeightLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    XLabel = new QLabel( groupBox1, "XLabel" );
    XLabel->setGeometry( QRect( 40, 50, 60, 21 ) );
    XLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    spinBox1 = new QSpinBox( groupBox1, "spinBox1" );
    spinBox1->setGeometry( QRect( 110, 50, 80, 21 ) );
    spinBox1->setMaxValue( 2048 );

    WidthLabel = new QLabel( groupBox1, "WidthLabel" );
    WidthLabel->setGeometry( QRect( 40, 130, 60, 21 ) );
    WidthLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    YLabel = new QLabel( groupBox1, "YLabel" );
    YLabel->setGeometry( QRect( 40, 90, 60, 21 ) );
    YLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    spinBox1_2 = new QSpinBox( groupBox1, "spinBox1_2" );
    spinBox1_2->setGeometry( QRect( 110, 130, 80, 21 ) );
    spinBox1_2->setMaxValue( 2048 );

    spinBox2 = new QSpinBox( groupBox1, "spinBox2" );
    spinBox2->setGeometry( QRect( 110, 90, 80, 21 ) );
    spinBox2->setMaxValue( 1200 );

    spinBox1_3 = new QSpinBox( groupBox1, "spinBox1_3" );
    spinBox1_3->setGeometry( QRect( 110, 170, 80, 21 ) );
    spinBox1_3->setMaxValue( 2048 );

    MaxLabel = new QLabel( groupBox1, "MaxLabel" );
    MaxLabel->setGeometry( QRect( 270, 50, 110, 21 ) );
    MaxLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    MinLabel = new QLabel( groupBox1, "MinLabel" );
    MinLabel->setGeometry( QRect( 270, 90, 110, 21 ) );
    MinLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    PointLabel = new QLabel( groupBox1, "PointLabel" );
    PointLabel->setGeometry( QRect( 270, 130, 110, 21 ) );
    PointLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    spinBox1_5 = new QSpinBox( groupBox1, "spinBox1_5" );
    spinBox1_5->setGeometry( QRect( 390, 50, 80, 21 ) );
    spinBox1_5->setMaxValue( 100 );
    spinBox1_5->setMinValue( -100 );
    spinBox1_5->setValue( 100 );

    spinBox1_4 = new QSpinBox( groupBox1, "spinBox1_4" );
    spinBox1_4->setGeometry( QRect( 390, 90, 80, 21 ) );
    spinBox1_4->setMaxValue( 100 );
    spinBox1_4->setMinValue( -100 );
    spinBox1_4->setValue( 0 );

    pushButton3 = new QPushButton( groupBox1, "pushButton3" );
    pushButton3->setGeometry( QRect( 390, 170, 80, 20 ) );
    pushButton3->setPaletteForegroundColor( QColor( 255, 255, 255 ) );
    pushButton3->setPaletteBackgroundColor( QColor( 0, 0, 0 ) );

    ColorLabel = new QLabel( groupBox1, "ColorLabel" );
    ColorLabel->setGeometry( QRect( 270, 170, 110, 20 ) );
    ColorLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    spinBox1_6 = new QSpinBox( groupBox1, "spinBox1_6" );
    spinBox1_6->setGeometry( QRect( 390, 130, 80, 21 ) );
    spinBox1_6->setMaxValue( 1000 );
    spinBox1_6->setMinValue( 0 );
    spinBox1_6->setValue( 50 );

// Sensor Group
    SensorBox = new QGroupBox( this, "SensorBox" );
    SensorBox->setGeometry( QRect( 20, 260, 561, 160 ) );

    useSens = new QCheckBox( SensorBox, "useSens" );
    useSens->setGeometry( QRect( 20, 20, 160, 21 ) );
    useSens->setChecked ( true );
    useSens->setEnabled( false );

    SensorCombo = new QComboBox( false, SensorBox, "SensorCombo" );
    SensorCombo->setEnabled( false );
    SensorCombo->setGeometry( QRect( 290, 20, 171, 21 ) );

    Format = new QLineEdit( SensorBox, "Format" );
    Format->setEnabled( false );
    Format->setGeometry( QRect( 102, 50, 150, 21 ) );

    Codec = new QLineEdit( SensorBox, "Codec" );
    Codec->setEnabled( false );
    Codec->setGeometry( QRect( 102, 90, 150, 21 ) );

    Line = new QLineEdit( SensorBox, "Line" );
    Line->setEnabled( false );
    Line->setGeometry( QRect( 400, 90, 150, 21 ) );

    Program = new QLineEdit( SensorBox, "Program" );
    Program->setEnabled( false );
    Program->setGeometry( QRect( 400, 50, 150, 21 ) );
    Program->setReadOnly( false );

    RDF = new QCheckBox( SensorBox, "RDF" );
    RDF->setGeometry( QRect( 20, 130, 160, 21 ) );
    RDF->setEnabled( false );

    LineLabel = new QLabel( SensorBox, "LineLabel" );
    LineLabel->setEnabled( false );
    LineLabel->setGeometry( QRect( 300, 90, 90, 21 ) );
    LineLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    ProgramLabel = new QLabel( SensorBox, "ProgramLabel" );
    ProgramLabel->setEnabled( false );
    ProgramLabel->setGeometry( QRect( 265, 50, 125, 21 ) );
    ProgramLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    FormatLabel = new QLabel( SensorBox, "FormatLabel" );
    FormatLabel->setEnabled( false );
    FormatLabel->setGeometry( QRect( 2, 50, 90, 21 ) );
    FormatLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    CodecLabel = new QLabel( SensorBox, "CodecLabel" );
    CodecLabel->setEnabled( false );
    CodecLabel->setGeometry( QRect( 2, 90, 90, 21 ) );
    CodecLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

    SensorLabel = new QLabel( SensorBox, "SensorLabel" );
    SensorLabel->setEnabled( false );
    SensorLabel->setGeometry( QRect( 190, 20, 80, 21 ) );
    SensorLabel->setAlignment( int( QLabel::AlignVCenter | QLabel::AlignRight ) );

// Global
    insCur = new QCheckBox( this, "insCur" );
    insCur->setGeometry( QRect( 20, 420, 300, 20 ) );

    pushButton2 = new QPushButton( this, "pushButton2" );
    pushButton2->setGeometry( QRect( 340, 440, 81, 31 ) );
    pushButton2->setIconSet( SmallIconSet( "cancel" ));

    pushButton1 = new QPushButton( this, "pushButton1" );
    pushButton1->setGeometry( QRect( 190, 440, 81, 31 ) );
    pushButton1->setIconSet( SmallIconSet( "ok" ));
    pushButton1->setDefault( TRUE );

    languageChange();
    UseSensor(true);

    // signals and slots connections
    connect( pushButton1, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( pushButton2, SIGNAL( clicked() ), this, SLOT( close() ) );
    connect( pushButton3, SIGNAL( clicked() ), this, SLOT( selColor() ) );
    connect( SensorCombo, SIGNAL( activated(const QString&) ), this, SLOT( ChooseProgram() ) );
}

/*
 *  Destroys the object and frees any allocated resources
 */
GraphDlg::~GraphDlg()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void GraphDlg::languageChange()
{
    // Global
    setCaption( tr2i18n( "Add Graph Meter" ) );
    pushButton1->setText( tr2i18n( "Ok" ) );
    pushButton2->setText( tr2i18n( "Cancel" ) );
    insCur->setText( tr2i18n( "Insert at cursor position" ) );
    // Sensor Group
    SensorBox->setTitle( tr2i18n( "Sensor" ) );
    useSens->setText( tr2i18n( "Use sensor" ) );
    SensorCombo->clear();
    SensorCombo->insertItem( tr2i18n( "CPU" ) );
    SensorCombo->insertItem( tr2i18n( "Memory" ) );
    SensorCombo->insertItem( tr2i18n( "Disk" ) );
    SensorCombo->insertItem( tr2i18n( "Network" ) );
    SensorCombo->insertItem( tr2i18n( "Time" ) );
    SensorCombo->insertItem( tr2i18n( "Uptime" ) );
    SensorCombo->insertItem( tr2i18n( "Sensor" ) );
    SensorCombo->insertItem( tr2i18n( "Textfile" ) );
    SensorCombo->insertItem( tr2i18n( "Noatun" ) );
    SensorCombo->insertItem( tr2i18n( "XMMS" ) );
    SensorCombo->insertItem( tr2i18n( "Program" ) );
    Format->setText( tr2i18n( "%v" ) );
    ProgramLabel->setText( tr2i18n( "Program:" ) );
    FormatLabel->setText( tr2i18n( "Format:" ) );
    SensorLabel->setText( tr2i18n( "Sensor:" ) );
    LineLabel->setText( tr2i18n( "Line:" ) );
    RDF->setText( tr2i18n( "RDF file" ) );
    CodecLabel->setText( tr2i18n( "Encoding:" ) );
    // Graph Group
    groupBox1->setTitle( tr2i18n( "Graph Parameters" ) );
    spinBox1->setPrefix( QString::null );
    spinBox1->setSuffix( tr2i18n( "px" ) );
    spinBox1_2->setPrefix( QString::null );
    spinBox1_2->setSuffix( tr2i18n( "px" ) );
    spinBox2->setPrefix( QString::null );
    spinBox2->setSuffix( tr2i18n( "px" ) );
    spinBox1_3->setPrefix( QString::null );
    spinBox1_3->setSuffix( tr2i18n( "px" ) );
    spinBox1_4->setPrefix( QString::null );
    spinBox1_4->setSuffix( QString::null );
    spinBox1_5->setPrefix( QString::null );
    spinBox1_5->setSuffix( QString::null );
    spinBox1_6->setPrefix( QString::null );
    spinBox1_6->setSuffix( QString::null );
    WidthLabel->setText( tr2i18n( "Width:" ) );
    HeightLabel->setText( tr2i18n( "Height:" ) );
    XLabel->setText( tr2i18n( "X axis:" ) );
    YLabel->setText( tr2i18n( "Y axis:" ) );
    MaxLabel->setText( tr2i18n( "Max Value:" ) );
    MinLabel->setText( tr2i18n( "Min Value:" ) );
    PointLabel->setText( tr2i18n( "Points:" ) );
    ColorLabel->setText( tr2i18n( "Color:" ) );
    pushButton3->setText( tr2i18n( "..." ) );
}

void GraphDlg::UseSensor( bool v ){
  SensorLabel->setEnabled(v);
  SensorCombo->setEnabled(v);
  Format->setEnabled(v);
  FormatLabel->setEnabled(v);
  LineLabel->setEnabled(v);
  Line->setEnabled(v);
  ProgramLabel->setEnabled(v);
  Program->setEnabled(v);
  CodecLabel->setEnabled(v);
  Codec->setEnabled(v);
  RDF->setEnabled(v);
  if(v){
    ChooseProgram();
  }
}

void GraphDlg::ChooseProgram(){
  if(SensorCombo->currentItem() == 0){
    ProgramLabel->setEnabled(false);
    Program->setEnabled(false);
    CodecLabel->setEnabled(false);
    Codec->setEnabled(false);
    FormatLabel->setEnabled(true);
    Format->setEnabled(true);
    Format->setText("%load");
    QToolTip::add( Format, tr2i18n( "%user<br>%system<br>%nice<br>%idle<br>%load (=%v)" ) );
    LineLabel->setEnabled(true);
    LineLabel->setText( tr2i18n( "CPU:" ) );
    Line->setEnabled(true);
    Line->setText("all");
    RDF->setEnabled(false);
  }
  else if(SensorCombo->currentItem() == 1){
    ProgramLabel->setEnabled(false);
    Program->setEnabled(false);
    CodecLabel->setEnabled(false);
    Codec->setEnabled(false);
    FormatLabel->setEnabled(true);
    Format->setEnabled(true);
    Format->setText("%fm");
    QToolTip::add( Format, tr2i18n( "%fm<br>%fmb<br>%um<br>%umb<br>%tm<br>%fs<br>%us<br>%ts" ) );
    LineLabel->setEnabled(false);
    Line->setEnabled(false);
    RDF->setEnabled(false);
  }
  else if(SensorCombo->currentItem() == 2){
    ProgramLabel->setEnabled(true);
    ProgramLabel->setText( tr2i18n( "MountPoint:" ) );
    Program->setEnabled(true);
    Program->setText("/");
    CodecLabel->setEnabled(false);
    Codec->setEnabled(false);
    FormatLabel->setEnabled(true);
    Format->setEnabled(true);
    Format->setText("%f");
    QToolTip::add( Format, tr2i18n( "%f<br>%fp<br>%u<br>%up<br>%t" ) );
    LineLabel->setEnabled(false);
    Line->setEnabled(false);
    RDF->setEnabled(false);
  }
  else if(SensorCombo->currentItem() == 3){
    ProgramLabel->setEnabled(true);
    ProgramLabel->setText( tr2i18n( "Device:" ) );
    Program->setEnabled(true);
    Program->setText("eth0");
    CodecLabel->setEnabled(false);
    Codec->setEnabled(false);
    FormatLabel->setEnabled(true);
    Format->setEnabled(true);
    Format->setText("%in");
    QToolTip::add( Format, tr2i18n( "%in<br>%out" ) );
    LineLabel->setEnabled(true);
    LineLabel->setText( tr2i18n( "Decimals:" ) );
    Line->setEnabled(true);
    Line->setText("0");
    RDF->setEnabled(false);
  }
  else if(SensorCombo->currentItem() == 4){
    ProgramLabel->setEnabled(false);
    Program->setEnabled(false);
    CodecLabel->setEnabled(false);
    Codec->setEnabled(false);
    FormatLabel->setEnabled(true);
    Format->setEnabled(true);
    Format->setText("dd.MM.yyyy");
    QToolTip::add( Format, tr2i18n( "d<br>dd<br>ddd<br>dddd<br>M<br>MM<br>MMM<br>MMMM<br>YY<br>YYYY<br>h<br>hh<br>m<br>mm<br>s<br>ss<br>z<br>zzz<br>AP<br>ap" ) );
    LineLabel->setEnabled(false);
    Line->setEnabled(false);
    RDF->setEnabled(false);
  }
  else if(SensorCombo->currentItem() == 5){
    ProgramLabel->setEnabled(false);
    Program->setEnabled(false);
    CodecLabel->setEnabled(false);
    Codec->setEnabled(false);
    FormatLabel->setEnabled(true);
    Format->setEnabled(true);
    Format->setText("%dd %h:%M");
    QToolTip::add( Format, tr2i18n( "%d<br>%h<br>%m<br>%s<br>%H<br>%M<br>%S" ) );
    LineLabel->setEnabled(false);
    Line->setEnabled(false);
    RDF->setEnabled(false);
  }
  else if(SensorCombo->currentItem() == 6){
    ProgramLabel->setEnabled(true);
    ProgramLabel->setText( tr2i18n( "Type:" ) );
    Program->setEnabled(true);
    Program->setText("temp2");
    CodecLabel->setEnabled(false);
    Codec->setEnabled(false);
    FormatLabel->setEnabled(true);
    Format->setEnabled(true);
    Format->setText("%v");
    QToolTip::add( Format, tr2i18n( "%v" ) );
    LineLabel->setEnabled(false);
    Line->setEnabled(false);
    RDF->setEnabled(false);
  }
  else if(SensorCombo->currentItem() == 7){
    ProgramLabel->setEnabled(true);
    ProgramLabel->setText( tr2i18n( "Path:" ) );
    Program->setEnabled(true);
    Program->setText("");
    CodecLabel->setEnabled(true);
    Codec->setEnabled(true);
    Codec->setText((QTextCodec::codecForLocale())->name());
    FormatLabel->setEnabled(true);
    Format->setEnabled(true);
    Format->setText("%v");
    QToolTip::add( Format, tr2i18n( "%v" ) );
    LineLabel->setEnabled(true);
    LineLabel->setText( tr2i18n( "Line:" ) );
    Line->setEnabled(true);
    Line->setText("");
    RDF->setEnabled(true);
  }
  else if(SensorCombo->currentItem() == 8){
    ProgramLabel->setEnabled(false);
    Program->setEnabled(false);
    CodecLabel->setEnabled(false);
    Codec->setEnabled(false);
    FormatLabel->setEnabled(true);
    Format->setEnabled(true);
    Format->setText("%id");
    QToolTip::add( Format, tr2i18n( "%id<br>%title<br>%time<br>%remain<br>%length<br>%full" ) );
    LineLabel->setEnabled(false);
    Line->setEnabled(false);
    RDF->setEnabled(false);
  }
  else if(SensorCombo->currentItem() == 9){
    ProgramLabel->setEnabled(false);
    Program->setEnabled(false);
    CodecLabel->setEnabled(true);
    Codec->setEnabled(true);
    Codec->setText((QTextCodec::codecForLocale())->name());
    FormatLabel->setEnabled(true);
    Format->setEnabled(true);
    Format->setText("%title");
    QToolTip::add( Format, tr2i18n( "%title<br>%time<br>%remain<br>%length<br>%full<br>%stitle<br>%artist<br>%album<br>%track<br>%cover<br>%rcover<br>%viewer<br>%rate<br>%freq<br>%channel<br>%monoster<br>%runquit<br>%vol<br>%volup<br>%voldown<br>%prev<br>%play<br>%pause<br>%stop<br>%next<br>%list<br>%ms" ) );
    LineLabel->setEnabled(false);
    Line->setEnabled(false);
    RDF->setEnabled(false);
  }
  else if(SensorCombo->currentItem() == 10){
    ProgramLabel->setEnabled(true);
    ProgramLabel->setText( tr2i18n( "Program:" ) );
    Program->setEnabled(true);
    CodecLabel->setEnabled(true);
    Codec->setEnabled(true);
    Codec->setText((QTextCodec::codecForLocale())->name());
    FormatLabel->setEnabled(true);
    Format->setEnabled(true);
    Format->setText("%v");
    QToolTip::add( Format, tr2i18n( "%v" ) );
    LineLabel->setEnabled(true);
    LineLabel->setText( tr2i18n( "Line:" ) );
    Line->setEnabled(true);
    Line->setText("");
    RDF->setEnabled(false);
  }
}

void GraphDlg::selColor(){
  int result = KColorDialog::getColor( graphColor );
   if ( result == KColorDialog::Accepted ){
      pushButton3->setPaletteBackgroundColor( graphColor );
      pushButton3->setText( QString::number(graphColor.red()) + "," + QString::number(graphColor.green()) + "," + QString::number(graphColor.blue()) );
   }
}

#include "graph.moc"
