/*
 * Copyright (C) 2003 Hans Karlsson <karlsson.h@home.se>
 */

#include "karamba.h"
karamba::karamba(QString fn, bool noquit) : QWidget( 0,"karamba", Qt::WGroupLeader | WStyle_Customize |  WRepaintNoErase| WStyle_NoBorder | WDestructiveClose  ){
    themePath = QFileInfo( fn ).dirPath(true);
    connect(qApp,SIGNAL(lastWindowClosed()),qApp,SLOT(quit()));
    connect( this, SIGNAL( karambaRun ( bool ) ), this, SLOT( status( bool ) ) );
    qApp->setGlobalMouseTracking( TRUE );
    MouseOver = false;
    nq = noquit;

    info = new NETWinInfo( qt_xdisplay(), winId(), qt_xrootwin(), NET::WMState );
    info->setDesktop( NETWinInfo::OnAllDesktops );
    info->setState( NETWinInfo::SkipTaskbar | NETWinInfo::SkipPager,NETWinInfo::SkipTaskbar | NETWinInfo::SkipPager );


    kWinModule = new KWinModule();
    connect( kWinModule,SIGNAL(currentDesktopChanged(int)), this,SLOT(currentDesktopChanged(int)) );


    kpop = new KPopupMenu( this );
    accColl = new KActionCollection( this );

    kpop->insertItem( SmallIconSet("reload"),tr2i18n("&Update"), this, SLOT(updateSensors()), Key_F5 );
    toggleLocked = new KToggleAction ( tr2i18n("&Lock position"), CTRL+Key_L, this, SLOT( slotToggleLocked() ), accColl, tr2i18n("Locked position") );
    accColl->insert(toggleLocked);
    toggleLocked->plug(kpop);

    kpop->insertSeparator();

    kpop->insertItem( SmallIconSet("fileopen"),tr2i18n("Add &new configuration"), this, SLOT(startNewKaramba()), CTRL+Key_O );
    kpop->insertItem( SmallIconSet("edit"),tr2i18n("&Edit configuration"),this,SLOT(editConfig()), CTRL+Key_E );
    kpop->insertItem( SmallIconSet("reload3"),tr2i18n("&Reload configuration"),this,SLOT(reloadConfig()), CTRL+Key_R );
    kpop->insertItem( SmallIconSet("fileclose"),tr2i18n("&Close this configuration"), this, SLOT(close()), CTRL+Key_D );
    kpop->insertItem( SmallIconSet("configure"),tr2i18n("C&onfigure Karamba"), this, SLOT(Config()), CTRL+Key_K );
    kpop->insertItem( SmallIconSet("configure"),tr2i18n("&Theme Creator"), this, SLOT(LaunchCreator()), CTRL+Key_T );
    kpop->insertItem( SmallIconSet("karamba"),tr2i18n("&About"), this, SLOT(about()), CTRL+Key_A );
    kpop->insertItem( SmallIconSet("about_kde"),tr2i18n("About &KDE"), this, SLOT(aboutKDE()), CTRL+Key_H );

    if( !nq ){
      kpop->insertSeparator();
      kpop->insertItem( SmallIconSet("exit"),tr2i18n("&Quit"), qApp, SLOT(quit()), CTRL+Key_Q );
    }

    kpop->polish();

    foundKaramba = false;
    onTop = false;
    fixedPosition = false;


    themeFile = fn;

    meterList = new QObjectList();
    meterList->setAutoDelete( true );
    sensorList = new QObjectList();
    sensorList->setAutoDelete( true );
    clickList = new QObjectList();
    timeList = new QObjectList();
    imageList = new QObjectList();
    iconList = new QPtrList<IconLabel>();


    client = kapp->dcopClient();
    if (!client->isAttached())
        client->attach();
    appId = client->registerAs(qApp->name());


    this->setBackgroundMode( NoBackground);
    if( !onTop){
        this->lower();
    }


    toggleHide();
    if( !parseConfig() ){
        setFixedSize(0,0);
        QTimer::singleShot( 100, this, SLOT(close()) );
        qDebug("false");
    }
    else{
        kroot = new KarambaRootPixmap((QWidget*)this);
        kroot->start();
        kroot->repaint( true );
    }
}

karamba::~karamba(){
    if(meterList != 0){
        meterList->clear();
        delete meterList;
    }

    if( sensorList != 0 ){
        sensorList->clear();
        delete sensorList;
    }

    if( imageList != 0 ){
        imageList->clear();
        delete imageList;
    }

    if( iconList != 0 ){
        iconList->clear();
        delete iconList;
    }

    if( clickList != 0 ){
      clickList->clear();
      delete clickList;
    }

    if( timeList != 0 ){
      timeList->clear();
      delete timeList;
    }

    if( toggleLocked != 0)
        delete toggleLocked;
    if( accColl != 0)
        delete accColl;

    if( kpop != 0)
        delete kpop;
    delete kWinModule;
}

bool karamba::parseConfig(){
    m_sysTimer = new QTimer(this);

    QFile file(themeFile);
    QString line;
    QString meter;
    int interval = 0;
    TextLabel *defaultTextLabel = 0;

    if ( file.open(IO_ReadOnly | IO_Translate) ){
        QTextStream t( &file );        // use a text stream
        QValueStack<QPoint> offsetStack;
        offsetStack.push(QPoint(0,0));

        int x=0;
        int y=0;
        int w=0;
        int h=0;
        while( ( line = t.readLine() ) !=0 ){
            QRegExp rx("^\\s*(\\S+)");
            rx.search(line);
            meter = rx.cap(1).upper();

            x = getInt("X",line)+offsetStack.top().x();
            y = getInt("Y",line)+offsetStack.top().y();
            w = getInt("W",line);
            h = getInt("H",line);


            if( meter == "KARAMBA" && !foundKaramba ){
                qDebug("karamba");
                toggleLocked->setChecked( getBoolean( "LOCKED", line ) );
                slotToggleLocked();
                QDesktopWidget *d = QApplication::desktop();


                hide = getString( "HIDE", line );

                x = ( x < 0 ) ? 0:x;
                y = ( y < 0 ) ? 0:y;
                themeX = x;
                themeY = y;
                if( w == 0 ||  h == 0){
                    w = 300;
                    h = 300;
                }
                themeW = w;
                themeH = h;

                if(hide != ""){
                  qDebug("hide: " + hide);
                  int limit = getInt( "lIMIT", line);
                  Hlimit = (limit != 0)?limit:20;
                }

                bool center = getBoolean("CENTER",line);
                if(center == true){
                  x = (d->width() - w)/2;
                  y = (d->height() - h)/2;
                }
                qDebug("X: " + QString::number(x) + " | Y: " + QString::number(y));
                move(x,y);


                setFixedSize(w,h);

                if( getBoolean( "RIGHT", line ) ){
                    x = d->width() - w;
                }
                else if( getBoolean( "LEFT", line ) ){
                        x = 0;
                }
                else if( getBoolean( "BOTTOM", line ) ){
                    y = d->height() - h;
                }
                else if( getBoolean( "TOP", line ) ){
                        y = 0;
                }

                pm = QPixmap(size());

                if( getBoolean("ONTOP", line ) ){
                    onTop = true;
                    KWin::setState( winId(), NET::StaysOnTop );
                }

                if( getBoolean("TOPBAR", line ) ){
                    move(x,0);
                    KWin::setStrut( winId(), 0, 0, h, 0 );
                    toggleLocked->setChecked( true );
                    slotToggleLocked();
                    toggleLocked->setEnabled(false);

                }

                if( getBoolean("BOTTOMBAR", line ) ){
                    int dh = d->height();
                    move( x, dh - h );
                    KWin::setStrut( winId(), 0, 0, 0, h );
                    toggleLocked->setChecked( true );
                    slotToggleLocked();
                    toggleLocked->setEnabled(false);
                }

                interval = getInt("INTERVAL",line);
                interval = ( interval == 0 ) ? 5000 : interval;
                foundKaramba = true;
            }

            if( meter == "THEME" ){
                QString path = getString( "PATH", line );
                QFileInfo info(path);
                if( info.isRelative()){
                    path = themePath +"/" + path;
                }
                qDebug("path: "+path);
                (new karamba( path, nq ))->show();
            }

            if( meter == "<GROUP>" ){
                int offsetX = offsetStack.top().x();
                int offsetY = offsetStack.top().y();
                offsetStack.push( QPoint( offsetX + getInt("X",line), offsetY + getInt("Y",line) ) );
            }

            if( meter == "</GROUP>" ){
                offsetStack.pop();
            }

            if( meter == "CLICKAREA" ){
                if( !hasMouseTracking() )
                    setMouseTracking(true);
                ClickArea *tmp = new ClickArea( x, y, w, h );
                tmp->setOnClick( getString( "ONCLICK", line ) );
                QString tiptext = getString("TOOLTIP",line);
                tmp->setKaramba(this);

                if (!tiptext.isEmpty()){
                  tmp->setTooltip(tiptext);
                }

                setSensor( line, (Meter*) tmp );
                clickList->append( tmp );
                if( getBoolean( "PREVIEW", line ) )
                    meterList->append( tmp );
            }

            // program sensor without a meter
            if(meter == "SENSOR=PROGRAM"){
                setSensor(line, 0 );
            }

            if(meter == "IMAGE"){
                QString file = getString("PATH",line);
                QString file_roll = getString("PATHROLL",line);
                int xon = getInt("XROLL",line);
                int yon = getInt("YROLL",line);
                QString tiptext = getString("TOOLTIP",line);
                bool center = getBoolean("CENTER",line);
                int w = getInt("W",line);
                int h = getInt("H",line);
                xon = ( xon <= 0 ) ? x:xon;
                yon = ( yon <= 0 ) ? y:yon;

                ImageLabel *tmp = new ImageLabel(x,y,0,0);
                tmp->setThemePath( themePath );
                tmp->setValue( file );
                tmp->setCenter( center, w, h );
                tmp->parseImages(file, file_roll, x,y, xon, yon);
                tmp->setKaramba(this);

                if (!tiptext.isEmpty()){
                  tmp->setTooltip(tiptext);
                }

                setSensor(line, (Meter*) tmp );
                meterList->append (tmp );
                imageList->append (tmp );
            }

            if(meter == "ICON"){
                QString file = getString("NAME",line);
                int roll = getInt("ROLL",line);
                int xon = getInt("XROLL",line);
                int yon = getInt("YROLL",line);
                QString tiptext = getString("TOOLTIP",line);
                bool center = getBoolean("CENTER",line);
                int s = getInt("S",line);
                int w = getInt("W",line);
                int h = getInt("H",line);
                xon = ( xon <= 0 ) ? x:xon;
                yon = ( yon <= 0 ) ? y:yon;
                if( !hasMouseTracking() && roll )
                    setMouseTracking(true);

                IconLabel *tmp = new IconLabel(x,y,0,0);
                tmp->setThemePath( themePath );
                if(file != ""){
                  tmp->setValue( file );
                }
                tmp->setSize( s );
                tmp->setCenter( center, w, h );
                tmp->parseImages(file, x,y, xon, yon, roll);
                tmp->setKaramba(this);

                if (!tiptext.isEmpty()){
                  tmp->setTooltip(tiptext);
                }

                setSensor(line, (Meter*) tmp );
                meterList->append (tmp );
                iconList->append (tmp );
            }

            if( meter == "DEFAULTFONT" ){
                int r,g,b;

                delete defaultTextLabel;
                defaultTextLabel = new TextLabel(x, y, w, h );

                get3Int( "COLOR", line, r, g, b );
                defaultTextLabel->setColor( r, g, b );
                get3Int( "BGCOLOR", line, r, g, b );
                defaultTextLabel->setBGColor( r, g, b );

                defaultTextLabel->setFont( getString( "FONT", line ) );
                defaultTextLabel->setFontSize( getInt( "FONTSIZE", line ) );
                defaultTextLabel->setAlignment( getString( "ALIGN", line ) );
                defaultTextLabel->setFixedPitch( getBoolean( "FIXEDPITCH", line ) );
                defaultTextLabel->setShadow( getInt( "SHADOW", line ) );
            }


            if(meter == "TEXT"){
                int r,g,b;

                TextLabel *tmp = new TextLabel(x, y, w, h );
                tmp->setKaramba(this);
                bool paramFound = false;

                get3Int( "COLOR", line, r, g, b, paramFound );
                if ( paramFound || !defaultTextLabel ){
                    tmp->setColor ( r, g, b );
                }
                else{
                    if ( defaultTextLabel ){
                        defaultTextLabel->getColor().rgb ( &r, &g, &b );
                        tmp->setColor( r, g, b );
                    }
                }

                get3Int( "BGCOLOR", line, r, g, b, paramFound );
                if ( paramFound  || !defaultTextLabel ){
                    tmp->setBGColor( r, g, b );
                }
                else{
                    if ( defaultTextLabel ){
                        defaultTextLabel->getBGColor().rgb ( &r, &g, &b );
                        tmp->setBGColor( r, g, b );
                    }
                }

                QString dFont = getString("FONT", line, paramFound);
                if( paramFound || !defaultTextLabel ){
                    tmp->setFont( dFont );
                }
                else{
                    if( defaultTextLabel ){
                        tmp->setFont( defaultTextLabel->getFont() );
                    }
                }


                int dFontSize = getInt( "FONTSIZE", line, paramFound );
                if( paramFound || !defaultTextLabel )
                    tmp->setFontSize( dFontSize );
                else
                    if( defaultTextLabel )
                        tmp->setFontSize( defaultTextLabel->getFontSize() );

                int dMove = getInt( "MOVE", line, paramFound );
                if( paramFound || !defaultTextLabel )
                    tmp->setMove( dMove );

                int dMaxMove = getInt( "MAXMOVE", line, paramFound );
                if( paramFound || !defaultTextLabel )
                    tmp->setMaxMove( dMaxMove );

                QString dAlign = getString( "ALIGN", line, paramFound );
                if( paramFound || !defaultTextLabel )
                    tmp->setAlignment( dAlign );
                else
                    if( defaultTextLabel )
                        tmp->setAlignment( defaultTextLabel->getAlignment() );

                bool dFp = getBoolean( "FIXEDPITCH", line, paramFound );
                if( paramFound || !defaultTextLabel )
                    tmp->setFixedPitch( dFp );
                else
                    if( defaultTextLabel )
                        tmp->setFixedPitch( defaultTextLabel->getFixedPitch() );

                int dShadow = getInt( "SHADOW", line, paramFound );
                if( paramFound || !defaultTextLabel )
                    tmp->setShadow( dShadow );
                else
                    if( defaultTextLabel )
                        tmp->setShadow( defaultTextLabel->getShadow() );

                tmp->setValue( getString( "VALUE", line ) );

                setSensor( line, (Meter*)tmp );
                meterList->append ( tmp );

            }

            if(meter == "BAR"){
                Bar *tmp = new Bar( x, y, w, h );
                tmp->setThemePath( themePath );
                tmp->setImage( getString( "PATH", line ) );
                tmp->setVertical( getBoolean( "VERTICAL", line ) );
                bool maxFound = false;
                int max = getInt( "MAX", line, maxFound );
                if( maxFound )
                    tmp->setMax( max );
                bool minFound = false;
                int min = getInt( "MIN", line, minFound );
                if( minFound )
                    tmp->setMin( min );
                tmp->setValue( getInt( "VALUE", line ) );
                setSensor( line, (Meter*)tmp );
                meterList->append ( tmp );
            }

            if(meter == "GRAPH"){
                int r, g, b;
                int points = getInt("POINTS",line);
                get3Int("COLOR", line, r, g, b);

                Graph *tmp = new Graph( x, y, w, h, points );
                bool maxFound = false;
                int max = getInt( "MAX", line, maxFound );
                if( maxFound )
                    tmp->setMax( max );
                bool minFound = false;
                int min = getInt( "MIN", line, minFound );
                if( minFound )
                    tmp->setMin( min );

                tmp->setColor( r , g, b );

                setSensor( line, (Graph*)tmp );
                meterList->append ( tmp );
            }
        }
        file.close();
    }
    if( !foundKaramba )
    {
        return false;
    }
    else
    {
        connect(m_sysTimer, SIGNAL(timeout()), SLOT(step()));
        emit karambaRun( true );
        MouseOver = false;
        toggleHide();
        interval = interval == 0 ? 1000 : interval;
        m_sysTimer->start(interval);
        QTimer::singleShot( 1000, this, SLOT(step()) );
        QTimer::singleShot( 5000, this, SLOT(step()) );
        QTimer::singleShot( 10000, this, SLOT(step()) );

        if( !onTop ){
            lowerTimer.start();
        }
        step();
        this->show();
        return true;
    }
}

void karamba::startNewKaramba(){
    QStringList fileNames;
    KSimpleConfig conf("karambarc",true);
    conf.setGroup("karamba");
    QString dDir =  conf.readEntry("ThemeDir", locateLocal("data", "karamba/themes/"));
    fileNames = KFileDialog::getOpenFileNames(dDir, "*.theme", 0, "Open configurations");
    for ( QStringList::Iterator it = fileNames.begin(); it != fileNames.end(); ++it ){
        QFileInfo file( *it );
        if( file.exists() ){
            (new karamba( *it, nq ))->show();
        }
    }
}

void karamba::Config(){
  KarambaConf *kc = 0;
  kc = new KarambaConf(0, "karconf", false, 0);
  kc->show();
}

void karamba::reloadConfig(){
    this->erase();
    m_sysTimer->disconnect();
    m_sysTimer = NULL;
    m_sysTimer = new QTimer(this);
    foundKaramba = false;
    onTop = false;
    fixedPosition = false;
    //if(meterList != 0){
      meterList = NULL;
      meterList = new QObjectList();
    //}
    //if( sensorList != 0 ){
      sensorList = NULL;
      sensorList = new QObjectList();
    //}
    //if( imageList != 0 ){
      imageList = NULL;
      imageList = new QObjectList();
    //}
    //if( iconList != 0 ){
      iconList = NULL;
      iconList = new QPtrList<IconLabel>();
    //}
    //if( clickList != 0 ){
      clickList = NULL;
      clickList = new QObjectList();
    //}
    //if( timeList != 0 ){
      timeList = NULL;
      timeList = new QObjectList();
    //}
    parseConfig();
}

void karamba::editConfig(){
    QFileInfo fileInfo( themeFile );
    QString path;

    if( fileInfo.isRelative() ){
        path = themePath + "/" + themeFile;
    }
    else{
        path = themeFile;
    }
    KProcess *proc = new KProcess;
    proc->setExecutable( "KarthemeCreator" );
    *proc << path;
    proc->start();

}


int karamba::getInt(QString w, QString &line ){
    bool b;
    return getInt( w, line, b );
}

int karamba::getInt(QString w, QString &line, bool &found ){
    QRegExp rx( "\\W+" + w +"=([-]?\\d+)", false );
    found = (rx.search(line)==-1)?false:true;
    return rx.cap(1).toInt();
}

void karamba::get3Int(QString w, QString &line, int &a, int &b, int &c ){
    bool bo;
    get3Int ( w, line, a, b, c, bo );
}

void karamba::get3Int(QString w, QString &line, int &a, int &b, int &c, bool &found ){
    QRegExp rx( "\\W+" + w + "=([-]?\\d+),([-]?\\d+),([-]?\\d+)", false );
    found = (rx.search(line)==-1)?false:true;
    a = rx.cap(1).toInt();
    b = rx.cap(2).toInt();
    c = rx.cap(3).toInt();
}

QString karamba::getString(QString w, QString &line){
    bool b;
    return getString ( w, line, b );
}

QString karamba::getString(QString w, QString &line, bool &found){
    QRegExp rx( "\\W+" + w + "=\"([^\"]*)\"", false );
    found = (rx.search(line)==-1)?false:true;
    if (rx.cap(1).isEmpty()){
        rx = QRegExp(w + "=(\\S+)", false);
        rx.search(line);
        return rx.cap(1);
    }
    else{
        return rx.cap(1);
    }

}

bool karamba::getBoolean( QString w, QString &line ){
    bool b;
    return getBoolean ( w, line, b );
}

bool karamba::getBoolean( QString w, QString &line, bool &found ){
    return  ( getString( w, line, found ).lower() == "true")?true:false;
}

void karamba::setSensor(QString &line, Meter* meter){
    Sensor* sensor = 0;

    QString sens = getString("SENSOR",line).upper();

    if( sens == "CPU" ){
        QString cpuNbr = getString( "CPU", line );
        sensor = sensorMap["CPU"+cpuNbr];
        if (sensor == 0){
            int interval = getInt("INTERVAL",line);
            interval = (interval == 0)?1000:interval;
            sensor = ( sensorMap["CPU"+cpuNbr] = new CPUSensor( cpuNbr, interval ) );
            sensorList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam("FORMAT",getString("FORMAT",line));
        sp->addParam("DECIMALS",getString("DECIMALS",line));
        sensor->addMeter(sp);
        sensor->setMaxValue(sp);
    }

    if( sens == "MEMORY" ){
        sensor = sensorMap["MEMORY"];
        if (sensor == 0){
            int interval = getInt("INTERVAL",line);
            interval = (interval == 0)?3000:interval;
            sensor = ( sensorMap["MEMORY"] = new MemSensor( interval ) );
            sensorList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam("FORMAT",getString("FORMAT",line));
        sensor->addMeter(sp);
        sensor->setMaxValue(sp);
    }


    if( sens == "DISK" ){
        sensor = sensorMap["DISK"];
        if (sensor == 0){
            int interval = getInt("INTERVAL",line);
            interval = (interval == 0)?5000:interval;
            sensor = ( sensorMap["DISK"] = new DiskSensor( interval ) );
            sensorList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        QString mntPt = getString("MOUNTPOINT",line);
        if( mntPt == ""  ){
            mntPt = "/";
        }
        sp->addParam("MOUNTPOINT",mntPt);
        sp->addParam("FORMAT",getString("FORMAT",line));
        sensor->addMeter(sp);
        sensor->setMaxValue(sp);
    }

    if( sens == "NETWORK"){
        int interval = getInt("INTERVAL",line );
        interval = (interval == 0)?2000:interval;
        QString device = getString("DEVICE", line );
        sensor = sensorMap["NETWORK"+device];
        if (sensor == 0){
            sensor = ( sensorMap["NETWORK"+device] = new NetworkSensor( device, interval ) );
            sensorList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam( "FORMAT", getString( "FORMAT", line ) );
        sp->addParam( "DECIMALS", getString( "DECIMALS", line ) );
        sensor->addMeter(sp);
    }

    if( sens == "UPTIME" ){
        sensor = sensorMap["UPTIME"];
        if (sensor == 0){
            int interval = getInt("INTERVAL",line);
            interval = (interval == 0)?60000:interval;
            sensor = ( sensorMap["UPTIME"] = new UptimeSensor( interval ));
            sensorList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam( "FORMAT", getString( "FORMAT", line ) );
        sensor->addMeter(sp);
    }

    if( sens == "SENSOR" ){
        sensor = sensorMap["SENSOR"];
        if (sensor == 0){
            int interval = getInt("INTERVAL",line);
            interval = (interval == 0)?30000:interval;
            sensor = ( sensorMap["SENSOR"] = new SensorSensor( interval ) );
            sensorList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam("FORMAT",getString("FORMAT",line));
        sp->addParam("TYPE",getString("TYPE",line));
        sensor->addMeter(sp);
    }


    if( sens == "TEXTFILE" ){
        QString path = getString( "PATH", line );
        bool rdf = getBoolean( "RDF", line );
        sensor = sensorMap["TEXTFILE"+path];
        if (sensor == 0){
            int interval = getInt( "INTERVAL", line );
            interval = ( interval == 0 )?60000:interval;
            QString encoding = getString( "ENCODING", line );

            sensor = ( sensorMap["TEXTFILE"+path] =
                           new TextFileSensor( path, rdf, interval, encoding ) );
            sensorList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam("LINE",QString::number(getInt("LINE",line)));
        sensor->addMeter(sp);
    }


    if( sens == "TIME"){
        sensor = sensorMap["DATE"];
        if (sensor == 0){
            int interval = getInt("INTERVAL",line);
            interval = (interval == 0)?60000:interval;
            sensor = ( sensorMap["DATE"] = new DateSensor( interval ) );
            sensorList->append( sensor );
            timeList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam("FORMAT",getString("FORMAT",line));
        sp->addParam("CALWIDTH",getString("CALWIDTH",line));
        sp->addParam("CALHEIGHT",getString("CALHEIGHT",line));
        sensor->addMeter(sp);
    }

#ifdef HAVE_XMMS

    if( sens == "XMMS" ){
        sensor = sensorMap["XMMS"];
        if (sensor == 0){
            int interval = getInt("INTERVAL",line);
            interval = (interval == 0)?1000:interval;
            QString encoding = getString( "ENCODING", line );
            int w = 0;
            int h = 0;
            w = getInt("W", line);
            h = getInt("H", line);

            sensor = ( sensorMap["XMMS"] = new XMMSSensor(  interval, encoding, w, h, this ) );
            sensorList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam("FORMAT",getString("FORMAT",line));
        sp->addParam("X",getString("X",line));
        sp->addParam("Y",getString("Y",line));
        sensor->addMeter(sp);
        sensor->setMaxValue(sp);
    }
#endif // HAVE_XMMS


    if( sens == "NOATUN" ){
        sensor = sensorMap["NOATUN"];
        if (sensor == 0){
            int interval = getInt("INTERVAL",line);
            interval = (interval == 0)?1000:interval;
            sensor = ( sensorMap["NOATUN"] = new NoatunSensor( interval, client ) );
            sensorList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam("FORMAT",getString("FORMAT",line));
        sensor->addMeter(sp);
        sensor->setMaxValue(sp);
    }

    if( sens == "PROGRAM"){
        QString progName = getString("PROGRAM",line);
        sensor = sensorMap["PROGRAM"+progName];
        if (sensor == 0){
            int interval = getInt("INTERVAL",line);
            interval = (interval == 0)?3600000:interval;
            QString encoding = getString( "ENCODING", line );
            sensor = (sensorMap["PROGRAM"+progName] = new ProgramSensor( progName, interval, encoding ) );
            sensorList->append( sensor );
        }
        SensorParams *sp = new SensorParams(meter);
        sp->addParam( "LINE", QString::number( getInt( "LINE", line ) ) );
        sp->addParam( "THEMAPATH", themePath );
        sensor->addMeter(sp);
    }

    if (sensor != 0){
        QTimer::singleShot( 0, sensor, SLOT(update()) );
        sensor->start();
    }
}


void karamba::passClick( QMouseEvent *e ){
    QObjectListIt it( *clickList ); // iterate over meters
    while ( it != 0 ){
        (( ClickArea* ) *it)->click( e );
        ++it;
    }

    QObjectListIt it2( *timeList ); // iterate over meters
    while ( it2 != 0 ){
        (( DateSensor* ) *it2)->toggleCalendar( e );
        ++it2;
    }
}

void karamba::mousePressEvent( QMouseEvent *e ){
    if( e->button() == RightButton ){
      kpop->exec(QCursor::pos());
    }
    else{
        clickPos = e->pos();
        clickX = e->x();
        clickY = e->y();
        if( locked ){
            passClick( e );
        }
        if( !onTop ){
            lower();
        }
    }
}


void karamba::mouseReleaseEvent( QMouseEvent *e ){
    clickPos = e->pos();
    qDebug("Click   X="+QString::number( clickPos.x() )+" Y="+QString::number( clickPos .y() ) );
    qDebug("Window  X="+QString::number( ( e->globalPos() - clickPos ).x())+" Y="+QString::number( ( e->globalPos() - clickPos ).y()));

    //update();
    step();
}

void karamba::mouseDoubleClickEvent( QMouseEvent *e ){
    if( !locked ){
        passClick( e );
    }
}


void karamba::mouseMoveEvent( QMouseEvent *e ){
  if( e->state() !=  0 && e->state() < 16 && !locked ){
      if(hide == "RIGHT"){
        themeY = e->globalY()  - clickY;
        move( QApplication::desktop()->width() - themeW, themeY );
      }
      else if(hide == "LEFT"){
        themeY = e->globalY() - clickY;
        move( 0 - themeW, themeY );
      }
      else if(hide == "TOP"){
        themeX = e->globalX() - clickX;
        move( themeX, 0 );
      }
      else if(hide == "BOTTOM"){
        themeX = e->globalX() - clickX;
        move( themeX, QApplication::desktop()->height() - themeH );
      }
      else{
        move( e->globalPos() - clickPos );
      }
  }
  else{
    // Change cursor over ClickArea
    QObjectListIt it( *clickList ); // iterate over meters
    bool insideArea = false;
    while ( it != 0 && !insideArea ){
        insideArea = (((ClickArea*) *it)->getRectangle()).contains(e->pos());
        ++it;
    }
    if(insideArea){
        if( cursor().shape() != PointingHandCursor ){
            setCursor( PointingHandCursor );
        }
    }
    else{
        if( cursor().shape() != ArrowCursor )
            setCursor( ArrowCursor );
    }

    QObjectListIt image_it( *imageList);	// iterate over image sensors
    while ( image_it != 0 ){
      ((ImageLabel*) *image_it)->rolloverImage(e);
      ++image_it;
    }

    QPtrListIterator<IconLabel> icon_it( *iconList);	// iterate over icon sensors
    IconLabel *icon;
    while ( (icon = icon_it.current()) != 0 ){
      icon->rolloverImage(e);
      ++icon_it;
    }
  }
}

void karamba::enterEvent( QEvent * ){
  MouseOver = true;
  toggleHide();
  setActiveWindow();
}

void karamba::leaveEvent( QEvent * ){
  MouseOver = false;
  toggleHide();
}

void karamba::toggleHide(){
    if(hide == "TOP"){
      if(MouseOver == true){
        if( y() != 0 ){
          updateSensors();
          move(themeX, 0);
          hidden = false;
        }
      }
      else if( y() == 0 ){
        move(themeX, 20 - themeH);
        hidden = true;
      }
  }
    if(hide == "BOTTOM"){
      if(MouseOver == true){
        if( y() != QApplication::desktop()->height() - themeH ){
          move(themeX, QApplication::desktop()->height() - themeH);
          hidden = false;
        }
      }
      else if( y() == QApplication::desktop()->height() - themeH ){
        move(themeX, QApplication::desktop()->height() - Hlimit);
        hidden = true;
      }
  }
    if(hide == "LEFT"){
      if(MouseOver == true){
        if( x() != 0 ){
          move(0, themeY);
          hidden = false;
        }
      }
      else if( x() == 0 ){
        move(Hlimit - themeW, themeY);
        hidden = true;
      }
  }
    if(hide == "RIGHT"){
      if(MouseOver == true ){
        if( x() != QApplication::desktop()->width() - themeW ){
          move(QApplication::desktop()->width() - themeW, themeY);
          hidden = false;
        }
      }
      else if( x() == QApplication::desktop()->width() - themeW ){
        move(QApplication::desktop()->width() - Hlimit, themeY);
        hidden = true;
      }
  }
}

void karamba::closeEvent ( QCloseEvent *  ){
    emit karambaRun( false );
    this->close(true);
    delete this;
}

void karamba::paintEvent ( QPaintEvent *e){
    if( !onTop && lowerTimer.elapsed() > 100 ){
        this->lower();
        lowerTimer.restart();
    }
    QRect rect = e->rect();
    bitBlt(this,rect.topLeft(),&pm,rect,Qt::CopyROP);
}

void karamba::updateSensors(){
    QObjectListIt it( *sensorList ); // iterate over meters
    while ( it != 0 ){
        ((Sensor*) *it)->update();
        ++it;
    }
    delete it;
    QTimer::singleShot( 500, this, SLOT(step()) );
}

void karamba::updateBackground(KSharedPixmap* kpm){
    background = QPixmap(*kpm);
    step();
}

void karamba::currentDesktopChanged( int ){
    kroot->repaint( true );
}



void karamba::step(){
    pm = QPixmap(size());
    QObjectListIt it( *meterList ); // iterate over meters
    p.begin(&pm);
    bitBlt(&pm,0,0,&background,0,Qt::CopyROP);
    while ( it != 0 ){
        ((Meter*) *it)->mUpdate(&p);
        ++it;
    }
    p.end();
    update();
}

void karamba::slotToggleLocked(){
    qDebug("toggleLocked");
    locked = toggleLocked->isChecked();
    if( locked ){
        toggleLocked->setIconSet(SmallIconSet("encrypted"));
        toggleLocked->setText( tr2i18n( "Un&lock position" ) );
    }
    else{
        toggleLocked->setIconSet(SmallIconSet("lockoverlay"));
        toggleLocked->setText( tr2i18n( "&Lock position" ) );
    }
}


void karamba::LaunchCreator(){
  KProcess *proc = new KProcess;
  proc->setExecutable( "KarthemeCreator" );
  proc->start();
}

void karamba::about(){
    KAboutApplication* aboutdlg;
    aboutdlg = new KAboutApplication();
    aboutdlg->show();
}

void karamba::aboutKDE(){
    KAboutKDE* aboutkdedlg;
    aboutkdedlg = new KAboutKDE();
    aboutkdedlg->show();
}

void karamba::status( bool r ){
  if( r == true ){
    qDebug("Karamba is running");
    isRunning = true;
  }
  else{
    qDebug("Karamba is halted");
    isRunning = false;
  }
}

#include "karamba.moc"
